/*
 * Part of the solution for Assignment 2 (CSC 218, Winter 2003), by
 * Stefan Bruda.
 */
#ifndef __BTREE_H
#define __BTREE_H

#include <iostream>  // for printtree
using namespace std;


/** Structure for nodes **/

struct node {
  char* info;
  node* parent;
  node* left;
  node* right;
  /*
   * For easy memory trace you can use a member variable like this:
   *
   *    char dummy [10240];
   *
   * This way you can easily see when one of your structures does not
   * get deallocated. :-)
   */
};
/*
 * A node also contains its parent (this field is empty for the root
 * node), because this will come in handy when implementing deltree
 * (in this module) as well as delnode (in module stree)
 */


/** Type for binary trees **/

typedef node* btree;


/** Constructors **/

/*
 * The empty tree
 */
const btree empty = 0;

/* 
 * Assemble arguments (info, left child, right child, parent) in a
 * node and returns a pointer to that node.  There is an extra
 * argument (the parent node), but this takes the appropriate default
 * value, so this function does conform to the specification.
 */
btree mknode(char*, btree = empty, btree = empty, btree = empty);


/** Accessors **/

/*
 * These functions return the various members of a node structure.
 */
char* key (btree);
btree lefttree (btree);
btree righttree (btree);
btree parent(btree);      /* not explicitly requested, but handy */


/** Destructor **/

/*
 * Deletes a subtree (second argument) of a tree (first argument).  To
 * dispose of a whole tree t do:
 *
 *   t = deltree(t,t);
 */
btree deltree (btree, btree);


/** Debug **/

/*
 * Pretty prints a tree (first argument), including information about
 * parent nodes.  Do not pass a second argument; this is used
 * internally (for proper indentation) in recursive calls.
 */
void printtree (btree, int = 0);  /* not explicitly requested, but handy */

#endif /* __BTREE_H */
