/*
 * Part of the solution for Assignment 2 (CSC 218, Winter 2003), by
 * Stefan Bruda.
 */
#include "stree.h"

/*
 * Accessor functions (lefttree, righttree, parent, key) are tested
 * throughout the code, and are in general too simple to be wrong.
 * mknode is also tested implicitly in, e.g., instree.  instree by the
 * way is tested within treesort.
 *
 * So it remains to test only the `delete' functions (namely deltree and
 * delnode) as well as memtree.
 */

int main (int argc, char** argv) {

  /*
   * Serach tree built by hand (just for the heck of it).
   */
  btree tt = mknode("5",
                    mknode("2",
                           mknode("1",NULL,NULL),
                           mknode("4",
                                  mknode("3",NULL,NULL),
                                  NULL)),
                    mknode("7",
                           mknode("6",NULL,NULL),
                           mknode("8",NULL,NULL)));
  printtree(tt);
  tt = deltree(tt,tt);
  
  /*
   * We start by building some tree (a search tree for that matter,
   * although some tests below do not use this property).  This also
   * provides further testing for instree.
   */
  cout << "============ test tree h ==================\n";
  btree h = instree("c",instree("m",instree("e",instree("b",instree("g",
            instree("u",instree("r",instree("f",empty))))))));
  /*
   * With the order of parameters required by the handout this would be:
   * 
   * btree h = instree(instree(instree(instree(instree(instree(instree(instree
   *             (empty,"f"),"r"),"u"),"g"),"b"),"e"),"m"),"c");
   *
   * I personally prefer the first version, and this is the reason I
   * changed the order of arguments.
   */

  printtree(h);
  /*
   * Look for an existent key (tests memtree).
   */
  cout << "++ memtree(\"r\",h) ++\n";
  cout << memtree("r",h);
  cout << "\n";
  /*
   * Look for non-existent keys (also tests memtree).
   */
  cout << "++ memtree(\"z\",h)  ++\n";
  cout << memtree("z",h);
  cout << "\n";
  cout << "++ instree (\"h\",h) ++\n";

  h = instree("h",h);
  printtree(h);
  /*
   * Delete a node with one child (also covers the case of deleting
   * nodes with no children, since there is no real difference between
   * these two cases).
   */
  cout << "++ delnode(h,memtree(\"m\",h)) ++\n";
  h = delnode(h,memtree("m",h));
  printtree(h);

  /*
   * Delete a node with two children, which also happens to be the
   * root (we thus test the alteration of the root node too).  This is
   * basically the comprehensive test of delnode (since we do two full
   * iterations).
   */
  cout << "++ delnode(h,memtree(\"f\",h)) ++\n";
  h = delnode(h,memtree("f",h));
  printtree(h);

  // clean up:
  h = deltree(h,h);

  /*
   * A degenerated tree.
   */
  cout << "============ test tree x ==================\n";
  btree x = instree("a",instree("b",instree("c",instree("d",empty))));
  printtree(x);

  /*
   * Delete nodes in the degenerated tree.  This is a rather common
   * output when we use the naive insertion implemented in this
   * assignment.
   */
  cout << "++ delnode(x,memtree(\"c\",x)) ++\n";
  x = delnode(x,memtree("c",x));
  printtree(x);

  cout << "++ delnode(x,memtree(\"d\",x)) ++\n";
  x = delnode(x,memtree("d",x));
  printtree(x);  

  // clean up:
  x = deltree(x,x);

  cout << "============ test of deltree: ==================\n";\
  btree y = instree("c",instree("m",instree("e",instree("b",instree("g",
            instree("u",instree("r",instree("f",empty))))))));
  printtree(y);

  /*
   * Test of deltree: node without children.
   */
  cout << "------ deltree \"m\" ------\n";
  y = deltree(y,memtree("m",y));
  printtree(y);

  /*
   * Delete a whole subtree.
   */
  cout << "------ deltree \"b\" ------\n";
  y = deltree(y,memtree("b",y));
  printtree(y);

  /*
   * Delete the root, i.e., the whole tree.  Note that this is also
   * the destructor for a tree.  It works correctly and does not leak
   * memory (tested using memprof 0.4.1).  The deleted tree is also
   * correctly set to empty.
   */
  cout << "------ deltree root (already tested above, but...) ------\n";
  y = deltree(y,y);
  printtree(y);

  /*
   * Finally, delete an empty tree.  This is not an error condition
   * (though the call does nothing), because it can happen in a call
   * to treesort with second argument 0.
   */
  cout << "------ deltree (empty) ------\n";
  y = deltree(y,y);
  printtree(y);
  
  // clean up done already!
}
