/*
 * Part of the solution for Assignment 2 (CSC 218, Winter 2003), by
 * Stefan Bruda.
 */
#include "treesort.h"

/*
 * Global index, set by treesort and used then by inorder.  See
 * function inorder for why it is better to use a global variable.  It
 * is not intended to be a public variable, and it is thus not
 * included in the header.
*/
int k;

/*
 * Traverses t in inorder and puts the result in strings.  Assumes
 * that strings has enough room for all the strings in t.  Returns the
 * number of elements written to strings.  Used by the main function
 * treesort below.  This is not intended to be a public function, so
 * it is not included in the header.
 */
int inorder(btree t, char** strings) {
  /*
   * We could use here a static local k instead of the global one, i.e.
   *
   * static int k = 0;
   *
   * the rest of the code is then unchanged.  BUT then treesort can be
   * called at most once per program, because k does not get set back
   * to 0 between calls to treesort (as it should).  So we use the
   * global k defined above, which will be initialized in function
   * treesort each time inorder is called.  Thanks to Lionel Perard (I
   * _think_, do correct me if I am wrong) for pointing out this
   * issue.
   */
  if (t == empty) return k;
  inorder(lefttree(t),strings);
  strings[k] = key(t);
  k++;
  inorder(righttree(t),strings);
  // we should never get here, but the C++ compiler gets fussy and
  // issues a warning without the certainty that return statement will
  // be reached, and I don't like warnings...
  return -1;
}

void treesort(char** strings, int nstrings) {
  btree st = empty;
  for (int i = 0; i < nstrings; i++)
    st = instree(strings[i],st);
  k = 0;
  inorder(st,strings);
  // clean up:
  st = deltree(st,st);
}
